@@ -18,6 +18,7 @@ import mock
 import operator
 import requests
 import six
+import urllib3
 
 from tripleo_common.image.exception import ImageUploaderException
 from tripleo_common.image import image_uploader
@@ -27,12 +28,12 @@ from tripleo_common.tests.image import fakes
 
 filedata = six.u(
     """container_images:
-    - imagename: docker.io/tripleoupstream/heat-docker-agents-centos:latest
+    - imagename: docker.io/tripleomaster/heat-docker-agents-centos:latest
       push_destination: localhost:8787
-    - imagename: docker.io/tripleoupstream/centos-binary-nova-compute:liberty
+    - imagename: docker.io/tripleomaster/centos-binary-nova-compute:liberty
       push_destination: localhost:8787
-    - imagename: docker.io/tripleoupstream/centos-binary-nova-libvirt:liberty
-    - imagename: docker.io/tripleoupstream/image-with-missing-tag
+    - imagename: docker.io/tripleomaster/centos-binary-nova-libvirt:liberty
+    - imagename: docker.io/tripleomaster/image-with-missing-tag
       push_destination: localhost:8787
 """)
 
@@ -70,22 +71,22 @@ class TestImageUploadManager(base.TestCase):
 
         dockerc = mockdocker.return_value
         dockerc.remove_image.assert_has_calls([
-            mock.call('docker.io/tripleoupstream'
+            mock.call('docker.io/tripleomaster'
                       '/centos-binary-nova-compute:liberty'),
-            mock.call('docker.io/tripleoupstream'
+            mock.call('docker.io/tripleomaster'
                       '/centos-binary-nova-libvirt:liberty'),
-            mock.call('docker.io/tripleoupstream'
+            mock.call('docker.io/tripleomaster'
                       '/heat-docker-agents-centos:latest'),
-            mock.call('docker.io/tripleoupstream'
+            mock.call('docker.io/tripleomaster'
                       '/image-with-missing-tag:latest'),
 
-            mock.call('localhost:8787/tripleoupstream'
+            mock.call('localhost:8787/tripleomaster'
                       '/centos-binary-nova-compute:liberty'),
-            mock.call('localhost:8787/tripleoupstream'
+            mock.call('localhost:8787/tripleomaster'
                       '/centos-binary-nova-libvirt:liberty'),
-            mock.call('localhost:8787/tripleoupstream'
+            mock.call('localhost:8787/tripleomaster'
                       '/heat-docker-agents-centos:latest'),
-            mock.call('localhost:8787/tripleoupstream/'
+            mock.call('localhost:8787/tripleomaster/'
                       'image-with-missing-tag:latest'),
         ])
 
@@ -134,10 +135,10 @@ class TestDockerImageUploader(base.TestCase):
         mock_process.returncode = 0
         mock_popen.return_value = mock_process
 
-        image = 'docker.io/tripleoupstream/heat-docker-agents-centos'
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
         tag = 'latest'
         push_destination = 'localhost:8787'
-        push_image = 'localhost:8787/tripleoupstream/heat-docker-agents-centos'
+        push_image = 'localhost:8787/tripleomaster/heat-docker-agents-centos'
 
         self.uploader.upload_image(image + ':' + tag,
                                    None,
@@ -159,10 +160,10 @@ class TestDockerImageUploader(base.TestCase):
 
     @mock.patch('subprocess.Popen')
     def test_upload_image_missing_tag(self, mock_popen):
-        image = 'docker.io/tripleoupstream/heat-docker-agents-centos'
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
         expected_tag = 'latest'
         push_destination = 'localhost:8787'
-        push_image = 'localhost:8787/tripleoupstream/heat-docker-agents-centos'
+        push_image = 'localhost:8787/tripleomaster/heat-docker-agents-centos'
 
         self.uploader.upload_image(image,
                                    None,
@@ -191,7 +192,7 @@ class TestDockerImageUploader(base.TestCase):
         mock_process.communicate.return_value = (json.dumps(result), '')
         mock_process.returncode = 0
         mock_popen.return_value = mock_process
-        image = 'docker.io/tripleoupstream/heat-docker-agents-centos'
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
         tag = 'latest'
         push_destination = 'localhost:8787'
 
@@ -291,9 +292,11 @@ class TestDockerImageUploader(base.TestCase):
         result = {
             'Labels': {
                 'rdo_version': 'a',
-                'build_version': '4.0.0'
+                'build_version': '4.0.0',
+                'release': '1.0.0',
+                'version': '20180125'
             },
-            'RepoTags': ['a']
+            'RepoTags': ['a', '1.0.0-20180125']
         }
         mock_process = mock.Mock()
         mock_process.communicate.return_value = (json.dumps(result), '')
@@ -301,12 +304,48 @@ class TestDockerImageUploader(base.TestCase):
         mock_popen.return_value = mock_process
 
         sr = image_uploader.SECURE_REGISTRIES
+        # simple label -> tag
         self.assertEqual(
             ('docker.io/t/foo', 'a'),
             image_uploader.discover_tag_from_inspect(
                 ('docker.io/t/foo', 'rdo_version', sr))
         )
 
+        # templated labels -> tag
+        self.assertEqual(
+            ('docker.io/t/foo', '1.0.0-20180125'),
+            image_uploader.discover_tag_from_inspect(
+                ('docker.io/t/foo', '{release}-{version}', sr))
+        )
+
+        # simple label -> tag with fallback
+        self.assertEqual(
+            ('docker.io/t/foo', 'a'),
+            image_uploader.discover_tag_from_inspect(
+                ('docker.io/t/foo:a', 'bar', sr))
+        )
+
+        # templated labels -> tag with fallback
+        self.assertEqual(
+            ('docker.io/t/foo', 'a'),
+            image_uploader.discover_tag_from_inspect(
+                ('docker.io/t/foo:a', '{releases}-{versions}', sr))
+        )
+
+        # Invalid template
+        self.assertRaises(
+            ImageUploaderException,
+            image_uploader.discover_tag_from_inspect,
+            ('docker.io/t/foo', '{release}-{version', sr)
+        )
+
+        # Missing label in template
+        self.assertRaises(
+            ImageUploaderException,
+            image_uploader.discover_tag_from_inspect,
+            ('docker.io/t/foo', '{releases}-{version}', sr)
+        )
+
         # no tag_from_label specified
         self.assertRaises(
             ImageUploaderException,
@@ -364,31 +403,74 @@ class TestDockerImageUploader(base.TestCase):
                 ('docker.io/t/baz', 'rdo_release', set())
             ])
 
-    @mock.patch('time.sleep')
-    def test_pull_retry(self, sleep_mock):
-        image = 'docker.io/tripleoupstream/heat-docker-agents-centos'
+    @mock.patch('tenacity.wait.wait_random_exponential.__call__')
+    def test_pull_retry(self, mock_wait):
+        mock_wait.return_value = 0
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
 
         dockerc = self.dockermock.return_value
         dockerc.pull.side_effect = [
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+            ['{"error": "ouch"}'],
+            ['{"error": "ouch"}'],
+            ['{"status": "done"}']
+        ]
+        self.uploader._pull(dockerc, image)
+
+        self.assertEqual(dockerc.pull.call_count, 5)
+        dockerc.pull.assert_has_calls([
+            mock.call(image, tag=None, stream=True)
+        ])
+
+    @mock.patch('tenacity.wait.wait_random_exponential.__call__')
+    def test_pull_retry_failure(self, mock_wait):
+        mock_wait.return_value = 0
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
+
+        dockerc = self.dockermock.return_value
+        dockerc.pull.side_effect = [
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+            urllib3.exceptions.ReadTimeoutError('p', '/foo', 'ouch'),
+        ]
+        self.assertRaises(urllib3.exceptions.ReadTimeoutError,
+                          self.uploader._pull, dockerc, image)
+
+        self.assertEqual(dockerc.pull.call_count, 5)
+        dockerc.pull.assert_has_calls([
+            mock.call(image, tag=None, stream=True)
+        ])
+
+    @mock.patch('tenacity.wait.wait_random_exponential.__call__')
+    def test_push_retry(self, mock_wait):
+        mock_wait.return_value = 0
+        image = 'docker.io/tripleoupstream/heat-docker-agents-centos'
+
+        dockerc = self.dockermock.return_value
+        dockerc.push.side_effect = [
             ['{"error": "ouch"}'],
             ['{"error": "ouch"}'],
             ['{"error": "ouch"}'],
             ['{"error": "ouch"}'],
             ['{"status": "done"}']
         ]
-        self.uploader._pull_retry(dockerc, image)
+        self.uploader._push(dockerc, image)
 
-        self.assertEqual(sleep_mock.call_count, 4)
-        dockerc.pull.assert_has_calls([
+        self.assertEqual(dockerc.push.call_count, 5)
+        dockerc.push.assert_has_calls([
             mock.call(image, tag=None, stream=True)
         ])
 
-    @mock.patch('time.sleep')
-    def test_pull_retry_failure(self, sleep_mock):
+    @mock.patch('tenacity.wait.wait_random_exponential.__call__')
+    def test_push_retry_failure(self, mock_wait):
+        mock_wait.return_value = 0
         image = 'docker.io/tripleoupstream/heat-docker-agents-centos'
 
         dockerc = self.dockermock.return_value
-        dockerc.pull.side_effect = [
+        dockerc.push.side_effect = [
             ['{"error": "ouch"}'],
             ['{"error": "ouch"}'],
             ['{"error": "ouch"}'],
@@ -396,10 +478,10 @@ class TestDockerImageUploader(base.TestCase):
             ['{"error": "ouch"}'],
         ]
         self.assertRaises(ImageUploaderException,
-                          self.uploader._pull_retry, dockerc, image)
+                          self.uploader._push, dockerc, image)
 
-        self.assertEqual(sleep_mock.call_count, 5)
-        dockerc.pull.assert_has_calls([
+        self.assertEqual(dockerc.push.call_count, 5)
+        dockerc.push.assert_has_calls([
             mock.call(image, tag=None, stream=True)
         ])
 
