@@ -31,6 +31,8 @@ _KNOWN_INTERFACE_FIELDS = [
                                          'power', 'raid', 'storage', 'vendor')
 ]
 
+CTLPLANE_NETWORK = 'ctlplane'
+
 
 class DriverInfo(object):
     """Class encapsulating field conversion logic."""
@@ -355,7 +357,8 @@ def register_ironic_node(node, client):
     ironic_node = client.node.create(**create_map)
 
     for mac in node.get("mac", []):
-        client.port.create(address=mac, node_uuid=ironic_node.uuid)
+        client.port.create(address=mac, physical_network=CTLPLANE_NETWORK,
+                           node_uuid=ironic_node.uuid)
 
     validation = client.node.validate(ironic_node.uuid)
     if not validation.power['result']:
@@ -631,7 +634,10 @@ def generate_hostmap(baremetal_client, compute_client):
         for port in baremetal_client.port.list(node=bm_node.uuid):
             hostmap[port.address] = {"compute_name": node.name,
                                      "baremetal_name": bm_node.name}
-    return hostmap
+    if hostmap == {}:
+        return None
+    else:
+        return hostmap
 
 
 def run_nova_cell_v2_discovery():
@@ -654,3 +660,15 @@ def get_node_profile(node):
         return capabilities_dict['profile']
 
     return None
+
+
+def get_node_hint(node):
+    """Return the 'capabilities:node' hint associated with the node """
+
+    capabilities = node.get('properties').get('capabilities')
+    capabilities_dict = capabilities_to_dict(capabilities)
+
+    if 'node' in capabilities_dict:
+        return capabilities_dict['node']
+
+    return None
